# cde.tcl --
#
#       package for making Tk apps look nice under CDE
#
#	It should be sufficent at the beginning of a wish app to simply:
#
#	    package require cde
#
#	The stuff below attempts to use the options database and the
#	various files under ~/.dt, $DTHOME, and /usr/dt to figure out
#	the user's current font and color set.  It then uses tk's 
#	palette routines to set sensible defaults, and then override
#	some of the options to try to make them look like CDE.
#
#	There really *must* be an easier way to get text background
#	colors and the radiobutton highlight colors out of the 
#	options database or winfo atom...  Unfortunately, I can't 
#	figure out how...
#
#	Any complaints, criticisms, and suggestions for improvement
#	can be sent to dhagberg@glatmos.com
#
# SCCS: %Z%%M% %I% %G% %U%
#
# Copyright (c) 1998 D. J. Hagberg
#
# See the file license.terms for information on usage and redistribution
# of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#

package require Tk
package provide cde 0.3.0

namespace eval ::cde {

    proc setup {} {
        variable \
            background       [option get . background background] \
            foreground       [option get . foreground foreground] \
            textfont         [option get . font font] \
            buttonfont       [option get . font font] \
            textbackground   white \
            textforeground   black \
            selectbackground black \
            selectforeground white \
            selectcolor      yellow \
            borderwidth      1 \
            priority         userDefault 

        # if any of these options are missing, we must not be under CDE, so bail
        if ![string length [set fg [option get . foreground foreground]]] return
        if ![string length [set bg [option get . background background]]] return

        if ![string length [set txtFont [option get . textFontList \
                textFontList]]] return
        if ![string length [set btnFont [option get . buttonFontList \
                buttonFontList]]] return
        set txtFont [lindex [split $txtFont :] 0]
        set btnFont [lindex [split $btnFont :] 0]
        set textfont $txtFont
        set buttonfont $btnFont
        #
        # If we can find the user's dt.resources file, we can find out the 
        # palette and text background/foreground colors
        #
        set txtBg $bg
        set txtFg $fg
        set selFg  white
        set selBg  black
        set selCol yellow
        set fh ""
        set palf ""
        set cur_rsrc ~/.dt/sessions/current/dt.resources
        set hom_rsrc ~/.dt/sessions/home/dt.resources
        if {[file readable $cur_rsrc] && [file readable $hom_rsrc]} {
            if {[file mtime $cur_rsrc] > [file mtime $hom_rsrc]} {
                if [catch {open $cur_rsrc r} fh] {set fh ""}
            } else {
                if [catch {open $hom_rsrc r} fh] {set fh ""}
            }
        } elseif {[file readable $cur_rsrc]} {
            if [catch {open $cur_rsrc r} fh] {set fh ""}
        } elseif {[file readable $hom_rsrc]} {
            if [catch {open $hom_rsrc r} fh] {set fh ""}
        }
        if [string length $fh] {
            while {[gets $fh ln] != -1} {
                regexp "^\\*0\\*ColorPalette:\[ \t]*(.*)\$" $ln nil palf
                regexp "^Window.Color.Background:\[ \t]*(.*)\$" $ln nil txtBg
                regexp "^Window.Color.Foreground:\[ \t]*(.*)\$" $ln nil txtFg
            }
            catch {close $fh}
            if {[string compare $txtBg $bg] != 0} {
                set selBg $txtFg
                set selFg $txtBg
            }
        }
        #
        # If the *0*ColorPalette setting was found above, try to find the
        # indicated file in ~/.dt, $DTHOME, or /usr/dt.  The 3rd line in the
        # file will be the radiobutton/checkbutton selectColor.
        #
        if [string length $palf] {
            set dtdir /usr/dt
            if [info exists env(DTHOME)] {
                set dtdir $env(DTHOME)
            }
            if [file readable ~/.dt/palettes/$palf] {
                set palf ~/.dt/palettes/$palf
            } elseif [file readable $dtdir/palettes/$palf] {
                set palf $dtdir/palettes/$palf
            } else {
                set palf ""
            }
            if [string length $palf] {
                if ![catch {open $palf r} fh] {
                    # selectColor will be the 3rd line in the file --
                    set ln ""; catch {gets $fh; gets $fh; gets $fh ln}
                    set ln [string trim $ln]
                    if [string length $ln] {set selCol $ln}
                    close $fh
                }
            }
        }
        #
        # Use the Tk procedure for setting all background/foreground colors
        #
        set background $bg
        set foreground $fg
        tk_setPalette background $bg foreground $fg
        #
        # Set default fonts and text field background/foreground
        #
        set textfont $txtFont
        foreach pref [list *Font *font] {
            option add $pref $txtFont $priority
        }
        set buttonfont $btnFont
        foreach pref [list *Button.font *Label.font *Menu*font *Menubutton.font\
                *Scale.font *Radiobutton.font *Checkbutton.font] {
            option add $pref $btnFont $priority
        }
        set textbackground $txtBg
        set textforeground $txtFg
        set selectbackground $selBg
        set selectforeground $selFg
        foreach pref [list *Entry *Listbox *Text] {
            option add $pref.background $txtBg $priority
            option add $pref.foreground $txtFg $priority
            option add $pref.selectBackground $selBg $priority
            option add $pref.selectForeground $selFg $priority
        }
        foreach pref [list *Entry *Text] {
            option add $pref.insertBackground $txtFg $priority
        }
        #
        # Set the Selector color for radiobuttons, checkbuttons, et. al
        #
        set selectcolor $selCol
        foreach pref [list *Checkbutton.selectColor *Radiobutton.selectColor \
                *Menu*selectColor *HighlightColor] {
            option add $pref $selCol $priority
        }
        #
        # Set the default *button borderwidth
        #
        foreach pref [list *Button *Radiobutton *Checkbutton] {
            option add $pref.activeBorderWidth $borderwidth
        }
        foreach pref [list *Button *Radiobutton *Checkbutton *Entry *Text \
                *Listbox *Scrollbar *Scale *Label] {
            option add $pref.borderWidth $borderwidth
        }
        option add *Scrollbar.elementBorderWidth $borderwidth
    }
}

cde::setup
